@@ -18,11 +18,17 @@ class TestHandler(Handler):
     'test:'. Primarily for testing purposes. """
 
     def handles_url(self, url):
+        """ Returns ``True`` if the URL starts with ``'test:'``. """
         if url.startswith('test:'):
             return url
         return None
 
     def initiate_auth(self, id_url, callback_uri, redir):
+        """
+        Immediately returns a :py:class:`disposition.Verified`, unless the URL
+        is ``'test:error'`` in which case it returns a
+        :py:class:`disposition.Error`.
+        """
         if id_url == 'test:error':
             return disposition.Error("Error identity requested", redir)
         return disposition.Verified(id_url, redir)
